#include "Mosaique.hpp"

static void remplirSHM(data*, uint8_t, int);

Mosaique::Mosaique()
		: nbLigne(2), nbColonne(2), pasLigne(hauteurStd / nbLigne),
		  pasColonne(largeurStd / nbColonne)
{
	initSHMServer();
}

Mosaique::~Mosaique()
{
	boost::interprocess::shared_memory_object::remove(nomSHM.c_str());
}

void Mosaique::initSHMServer()
{
	size_t tailleSHM = tailleData * 8;
	genererNomSHMMosaique();
	printf("Taille de la memoire partagee : %d\n", (int)tailleSHM);
	try{
		segment = new 
				boost::interprocess::managed_shared_memory(
						boost::interprocess::create_only, 
						nomSHM.c_str(), 
						tailleSHM);

		data d;
		d.width =		largeurStd;
		d.height =		hauteurStd;
		d.nbCanaux =	nbCanaux;
		d.numImage =	0;
		output = segment->construct<data>
				("video")//name of the object
				(d);
	}catch(boost::interprocess::interprocess_exception e){
		BOOST_LOG_TRIVIAL(debug) << "ServerSHM.cpp " <<
				e.what();
		exit(EXIT_FAILURE);
	}
	remplirSHM(output, 255, tailleSHM / 8);
}

void Mosaique::genererNomSHMMosaique()
{
	nomSHM = "camera100";
}

data* Mosaique::getPtrMosaique()const
{
	return output;
}

/**
 * Rajoute un client qui va fournir une des videos qui vont servir a generer 
 * la mosaique.
 * @param  nv client a ajouter
 * @return    Vrai si l'ajout s'est bien deroule. Faux sinon (deja 4 clients pour
 *            cette mosaique).
 */
bool Mosaique::addClient(SHMClient* nv)
{
	if(clients.size() >= 4)
		return false;
	clients.push_back(nv);
	return true;
}

bool Mosaique::genererMosaique()
{
	int i =0;
	for(int h = 0 ; h < hauteurStd ; ++h){
		for(int w = 0 ; w < largeurStd ; ++w){
			//definition de l'image a modifier
			int numImage = numImagePixel(w, h);
			SHMClient* cl = clients[numImage];
			printf("on est ici %d\n", i);++i;

			//Transfert du pixel
			int numPixel = h * largeurStd + w;
			output->data[numPixel] = cl->getPtrDonnees()->data[numPixel];
		}
	}
	return true;
}


/**
 * Determine a quelle image source correspond le pixel dont les coordonnees 
 * sont passees en parametre.
 * @param  colPixel numero de colonne du pixel de la mosaique.
 * @param  ligPixel numero de ligne du pixel de la mosaique.
 * @return          identifiant de l'image source a laquelle correspond le pixel.
 */
int Mosaique::numImagePixel(int colPixel, int ligPixel)const
{
	if(colPixel < (largeurStd / 2)){
		//si dans la colonne 1
		if(ligPixel < (hauteurStd / 2))
			return 1;
		else
			return 3;
	}else{
		//si dans la colonne 2
		if(ligPixel < (hauteurStd / 2))
			return 2;
		else
			return 4;
	}
}

/**
 * Retourne le numero du pixel de l'image identifiee par numImage a afficher
 * sur la mosaique aux coordonnees colPixel et ligPixel.
 * @param  colPixel numeros de colonne du pixel de la mosaique que l'on souhaite 
 *                  remplir.
 * @param  ligPixel numeros de ligne du pixel de la mosaique que l'on souhaite
 *                  remplir.
 * @param  numImage numeros de l'image dont on va extraire le pixel que l'on va
 *                  afficher sur la mosaique.
 * @return          le numero du pixel de l'image numImage a afficher sur la 
 *                  mosaique.
 */
int Mosaique::pixelImage(int colPixel, int ligPixel, int numImage)const
{
	//reperage du pixel dans sa case
	switch(numImage){
	case 1:
		break;
	case 2:
		colPixel -= largeurStd / 2;
		break;
	case 3:
		ligPixel -= hauteurStd / 2;
	case 4:
		colPixel -= largeurStd / 2;
		ligPixel -= hauteurStd / 2;
	}

	//equivalent entre num pixel de case et numpixel de l'image.
	int ligPixelImg = ligPixel * 2;
	int colPixelImg = colPixel * 2;

	return ligPixelImg * largeurStd + colPixelImg;
}

void remplirSHM(data* d, uint8_t couleur, int taille)
{
	memset(d->data,
			couleur, 
			taille);
}






// int Mosaique::pixelImage(int colPixel, int ligPixel, int numImage)const
// {
// 	//numero de ligne de l'image
// 	int numLig = ((numImage - 1) / nbLigne) + 1;
// 	//numero de colonne de l'image
// 	int numCol = ((numImage - 1) % nbLigne) + 1;

// 	//coordonnes de la case de la mosaique correspondant a l'image
// 	int ligCM = ligPixel - numLig * (hauteurStd / nbLigne);
// 	int colCM = colPixel - numCol * (hauteurStd / nbLigne);

// 	//nombre de pixel qui separent une colonne de la mosaique d'une autre
// 	int pasLigne = hauteurStd / nbLigne;
// 	//nombre de pixel qui separent une ligne de la mosaique d'une autre
// 	int pasColonne = largeurStd / nbColonne;

// 	//calcul du resultat final
// 	int nbPixelDebutCase = (ligCM - 1) * pasLigne * largeurStd + 
// 			pasColonne * colCM;
// 	int nbPixelDansCase = ligCM * pasColonne * (nbLigne - 1) + colCM;
	

// 	return nbPixelDansCase;
// }