#ifndef MOSAIQUE_HPP
#define MOSAIQUE_HPP

#define BOOST_ALL_DYN_LINK
#include <boost/log/trivial.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>

#include "SHMClient.hpp"
#include "data.hpp"

class CaseMosaique;

/**
 * Prend des images en entree et cree une mosaique de toutes ces images en sortie.
 * Toutes les images doivent etre au format standard (defini dans data.hpp).
 * La sortie sera elle aussi au format standard.
 * Numerotation des images affichees :
 * 				+-+-+
 * 				|1|2|
 * 				+-+-+
 * 				|3|4|
 * 				+-+-+
 */
class Mosaique
{
public:
	Mosaique();
	~Mosaique();

	bool addClient(SHMClient*);
	data* getPtrMosaique()const;
	bool genererMosaique();
	
protected:
	void initSHMServer();
	void genererNomSHMMosaique();
	int numImagePixel(int, int)const;
	int pixelImage(int, int, int)const;

	/**
	 * Listes des clients qui contiennent les images qui vont etre utilisees pour
	 * generer la mosaique.
	 */
	std::vector<SHMClient*> clients;
	
	/**
	 * Segment de memoire partagee ou va etre place l'image mosaique generee.
	 */
	boost::interprocess::managed_shared_memory* segment;

	/**
	 * Pointeur vers le donnees generees par la mosaique.
	 */
	data* output;

	/**
	 * Nom de la memoire partagee dans laquelle va etre stockee les donnees generees par
	 * la mosaique.
	 */
	std::string nomSHM;
	
	/**
	 * Nombre de ligne de la mosaique.
	 */
	const int nbLigne;

	/**
	 * Nombre de colonne de la mosaique.
	 */
	const int nbColonne;

	/**
	 * Nombre de pixel qui separent une ligne de pixel d'une autre
	 * hauteurStd / nbLigne
	 */
	const int pasLigne;
	
	/**
	 * Nombre de pixel qui separent une colonne de la mosaique d'une autre
	 * largeurStd / nbColonne
	 */
	const int pasColonne;

};

#endif /*MOSAIQUE_HPP*/