#ifndef SHMCLIENT_HPP
#define SHMCLIENT_HPP

#define BOOST_ALL_DYN_LINK
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/log/trivial.hpp>
#include <boost/asio.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/bind.hpp>
#include <boost/thread/thread.hpp>

#include <opencv2/core/core.hpp>
#include <opencv2/core/core_c.h>
#include <opencv2/highgui/highgui.hpp>
#include <opencv/highgui.h>

#include <cstdlib> //std::system
#include <cstddef>
#include <cassert>
#include <utility>

#include "data.hpp"

class ArgumentsSHMClient
{
public:
	int udpPort;
	int udpUrl;
	int fps;
	int camId;
};

/**
 * Classe qui va lire la memoire partagee.
 */
class SHMClient
{
public:
	SHMClient(int argc, char *argv[]);
	~SHMClient();

	std::string getNomSHM()const;
	data* getPtrDonnees()const;
	int getFPS()const;

	void init(int argc, char *argv[]);
	boost::interprocess::managed_shared_memory* ouvrirMemoire(std::string);
	ArgumentsSHMClient* parseArgument(int argc, const char* const argv[]);

protected:
	/**
	 * Nom de la memoire partagee qui va etre lue par ce client.
	 */
	std::string nomSHM;

	/**
	 * Donnees de la memoire partagee.
	 */
	data* d;

	/**
	 * FPS determime par l'utilisateur dans les options.
	 */
	int fps;

	/**
	 * Id de la camera en entree. Option definie par l'utilisateur.
	 */
	int camId;

	/**
	 * Classe qui gere la memoire partagee.
	 */
	boost::interprocess::managed_shared_memory* segment;
};

#endif /*SHMCLIENT_HPP*/