#ifndef SERVERSHM_HPP
#define SERVERSHM_HPP

#include <string>

#define BOOST_ALL_DYN_LINK
#include <boost/log/trivial.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>

#include <opencv2/core/core_c.h>

#include "data.hpp"
#include "Capture.hpp"
#include "FluxVideoModifie.hpp"


class ServerSHM
{
public:
	ServerSHM(int argc, const char* const argv[]);
	~ServerSHM();

protected:
	void parseArgument(int argc, const char* const argv[]);
	void initVideo();
	void initSHM();
	void lancerLecture();

	/**
	 * Adresse a laquelle se trouve l'entree video.
	 */
	std::string url;

	/**
	 * Largeur de l'image en sotie.
	 */
	int width;

	/**
	 * Hauteur de l'image en sortie.
	 */
	int height;

	/**
	 * Numero associe a la sortie camera.
	 */
	int camId;

	/**
	 * Nom de la memoire partagee qui sera geree par ce server.
	 */
	std::string nomSHM;

	/**
	 * La memoire partagee geree par ce server.
	 */
	boost::interprocess::managed_shared_memory segment;

	/**
	 * Classe de capture de la video.
	 */
	Capture* video;

	/**
	 * Classe de modification de la video.
	 * TODO ne l'instancier que quand c'est nescessaire.
	 */
	FluxVideoModifie*	fm;

	/**
	 * Vrai si la video doit etre transformee au format opencv
	 */
	bool convOpencv;	

	/**
	 * Objet qui se trouve dans la memoire partagee.
	 * C'est en modifiant cet objet que l'on peut communiauer
	 * avec les autres processus.
	 */
	data* objetPartage;
};

#endif /*fin SERVERSHM_HPP*/