#include "ffmpegTools.hpp"

void ffmpegTools::print_info( AVFormatContext *c )
{
	printf("print_info AVFormatContext :\n");

	// Type input/output
	if( c->iformat )
	{
		printf("\t AVInputFormat : %s.\n", c->iformat->name );
	}
	if( c->oformat )
	{
		printf("\t AVOutputFormat : %s.\n", c->oformat->name );
	}

	if( c->pb )
		printf("\t AVIOContext : ok.\n" );

	printf("\t filename : %s\n", c->filename );

	// print AVStream info
		printf("\t nb_streams : %i\n", c->nb_streams );
		if( c->nb_streams>0 && c->streams[0] )
			print_info( c->streams[0] );

	printf("\t max_picture_buffer : %i\n", c->max_picture_buffer );
	AVStream *video_st = c->streams[0];
	printf("\t fps : %.2f\n", av_q2d(video_st->r_frame_rate) );


	return;
}

void ffmpegTools::print_info( AVCodecContext *c )
{
	printf("  - Codec infos :\n");
	AVCodec *p=NULL;

	// type VIDEO / AUDIO
		if( c->coder_type == AVMEDIA_TYPE_VIDEO )
			printf("\t coder_type : VIDEO\n" );
		if( c->coder_type == AVMEDIA_TYPE_AUDIO )
			printf("\t coder_type : AUDIO\n" );

	// codec name, tag
		p = avcodec_find_encoder(c->codec_id);
		if( !p )
			p = avcodec_find_decoder(c->codec_id);
		if( p )
		{
			printf("\t codec : %s\n", p->name );
			// Codec Profile
			const AVProfile *prof;
			if (c->profile == FF_PROFILE_UNKNOWN || !p->profiles)
				printf("\t profile : None\n" );
			else
				for (prof = p->profiles; prof->profile != FF_PROFILE_UNKNOWN; prof++)
					if (prof->profile == c->profile)
						printf("\t profile : %s\n", prof->name );
		}
		else
		{
			if (c->codec_id == CODEC_ID_MPEG2TS)
			{
				/* fake mpeg2 transport stream codec (currently not registered) */
				printf("\t codec : mpeg2ts\n" );
			}
			else if (c->codec_name[0] != '\0')
			{
				printf("\t codec : %s\n", c->codec_name );
			}
			else
			{
				/* output avi tags */
				char buf1[32];
				if (c->codec_type == AVMEDIA_TYPE_VIDEO)
				{
					snprintf(buf1, sizeof(buf1), "%c%c%c%c",
							 c->codec_tag & 0xff,
							 (c->codec_tag >> 8) & 0xff,
							 (c->codec_tag >> 16) & 0xff,
							 (c->codec_tag >> 24) & 0xff);
				} else {
					snprintf(buf1, sizeof(buf1), "0x%04x", c->codec_tag);
				}
				printf("\t codec : %s\n", buf1 );
			}
		}

	//        printf("\t codec_tag : %c%c%c%c\n", c->codec_tag << 24, c->codec_tag << 16, c->codec_tag << 8, c->codec_tag);
	//        printf("\t codec_name : %s\n", c->codec_name );



	// pixel format, size
		printf("\t size : %ix%i\n", c->width, c->height );
		if( c->pix_fmt == PIX_FMT_YUV420P )
			printf("\t pix_fmt : PIX_FMT_YUV420P\n" );
		else
			printf("\t pix_fmt : %i\n", c->pix_fmt );

	// codec params
		printf("\t bit_rate : %i\n", c->bit_rate );
		printf("\t timebase : %i/%i\n", c->time_base.num, c->time_base.den );
		//printf("\t fps : %0.2f\n", (float)c->frame_rate / c->frame_rate_base);

	// SAR / DAR
		AVRational display_aspect_ratio;
		if (c->sample_aspect_ratio.num)
		{
			printf("\t sample_aspect_ratio (SAR): %d:%d\n", c->sample_aspect_ratio.num, c->sample_aspect_ratio.den );
			av_reduce(&display_aspect_ratio.num, &display_aspect_ratio.den,
					  c->width * c->sample_aspect_ratio.num,
					  c->height * c->sample_aspect_ratio.den,
					  1024 * 1024);
			printf("\t display_aspect_ratio (DAR): %d:%d\n", display_aspect_ratio.num, display_aspect_ratio.den);
		}

		printf("\t q=%i-%i\n", c->qmin, c->qmax );
		printf("\t gop_size : %i\n", c->gop_size );
		printf("\t min gop size : %i\n", c->keyint_min );
		printf("\t max_b_frames : %i\n", c->max_b_frames );
		printf("\t b_frame_strategy (best nb B-frames) : %i\n", c->b_frame_strategy );
		printf("\t refs : %i\n", c->refs );
		printf("\t i_quant_factor : %lf\n", c->i_quant_factor );

    // is experimental?
//    if( m_pOutputCodec->capabilities & CODEC_CAP_EXPERIMENTAL )
//        printf("\t experimental : yes.\n");
//    else
//        printf("\t experimental : no.\n");

    return;
}

void ffmpegTools::print_info(AVPacket *c )
{
	printf("  - Packet infos :\n");

	printf("\t durée : %d\n", c->duration);
	printf("\t taille des données : %d\n", c->size);
	printf("\t taille : %d\n", c->size);

	if(c->data == NULL){
		printf("\t attention, il n'y a pas de données associées !\n");
	}

	if(c->side_data == NULL){
		printf("\t pas de side_data\n");
	}else{
		printf("\t side_data : taille %d\n", c->side_data->size);
	}
}

std::string ffmpegTools::getStringError( int error )
{
	char buf[512];
	av_strerror( error, &buf[0], sizeof(buf) );
	return std::string(buf);

}

void ffmpegTools::getRGBColor(const AVFrame& frame, int posX, int posY, unsigned char &r, unsigned char &g, unsigned char &b)
{
	// Y component
	const unsigned char y = frame.data[0][frame.linesize[0]*posY + posX];

	// U, V components
	posX /= 2;
	posY /= 2;
	const unsigned char u = frame.data[1][frame.linesize[1]*posY + posX];
	const unsigned char v = frame.data[2][frame.linesize[2]*posY + posX];

	// RGB conversion
	r = y + 1.402*(v-128);
	g = y - 0.344*(u-128) - 0.714*(v-128);
	b = y + 1.772*(u-128);

	return;
}

//void ffmpegTools::print_info( AVCodecContext *c, vizILogger *pLog )
//{
//    pLog->Log("  - Codec infos :");
//    AVCodec *p=NULL;
//
//    // type VIDEO / AUDIO
//        if( c->coder_type == AVMEDIA_TYPE_VIDEO )
//            pLog->Log("   media type : VIDEO" );
//        if( c->coder_type == AVMEDIA_TYPE_AUDIO )
//            pLog->Log("   media type : AUDIO" );
//
//    // recherche du codec dans encoder/decoder
//        p = avcodec_find_decoder(c->codec_id);
//        if( !p )
//            p = avcodec_find_encoder(c->codec_id);
//
//    // codec name, tag
//        if( p )
//        {
//            pLog->Log("   codec : %s", p->name );
//
//            // Codec Profile
//            const AVProfile *prof;
//            if (c->profile == FF_PROFILE_UNKNOWN || !p->profiles)
//                pLog->Log("   profile : None" );
//            else
//                for (prof = p->profiles; prof->profile != FF_PROFILE_UNKNOWN; prof++)
//                    if (prof->profile == c->profile)
//                        pLog->Log("   profile : %s", prof->name );
//        }
//        else
//        {
//            if (c->codec_id == CODEC_ID_MPEG2TS)
//            {
//                /* fake mpeg2 transport stream codec (currently not registered) */
//                pLog->Log("   codec : mpeg2ts" );
//            }
//            else if (c->codec_name[0] != '\0')
//            {
//                pLog->Log("   codec : %s", c->codec_name );
//            }
//            else
//            {
//                /* output avi tags */
//                char buf1[32];
//                if (c->codec_type == AVMEDIA_TYPE_VIDEO)
//                {
//                    snprintf(buf1, sizeof(buf1), "%c%c%c%c",
//                             c->codec_tag & 0xff,
//                             (c->codec_tag >> 8) & 0xff,
//                             (c->codec_tag >> 16) & 0xff,
//                             (c->codec_tag >> 24) & 0xff);
//                }
//                else
//                {
//                    snprintf(buf1, sizeof(buf1), "0x%04x", c->codec_tag);
//                }
//                pLog->Log("   codec : %s", buf1 );
//                // pLog->Log("\t codec_tag : %c%c%c%c\n", c->codec_tag<<24, c->codec_tag<<16, c->codec_tag<<8, c->codec_tag);
//            }
//        }
//
//    // pixel format, size
//        pLog->Log("   size : %ix%i", c->width, c->height );
//        if( c->pix_fmt == PIX_FMT_YUV420P )
//            pLog->Log("   pix_fmt : PIX_FMT_YUV420P" );
//        else
//            pLog->Log("   pix_fmt : %i", c->pix_fmt );
//
//    // codec params
//        pLog->Log("   bit_rate : %i", c->bit_rate );
//        pLog->Log("   timebase : %i/%i", c->time_base.num, c->time_base.den );
//        pLog->Log("   fps : %0.2f", 1.0 / av_q2d(c->time_base));
//
//    // SAR / DAR
//        AVRational display_aspect_ratio;
//        if (c->sample_aspect_ratio.num)
//        {
//            pLog->Log("   sample_aspect_ratio (SAR): %d:%d", c->sample_aspect_ratio.num, c->sample_aspect_ratio.den );
//            av_reduce(&display_aspect_ratio.num, &display_aspect_ratio.den,
//                      c->width * c->sample_aspect_ratio.num,
//                      c->height * c->sample_aspect_ratio.den,
//                      1024 * 1024);
//            pLog->Log("   display_aspect_ratio (DAR): %d:%d", display_aspect_ratio.num, display_aspect_ratio.den);
//        }
//        pLog->Log("   q=%i-%i", c->qmin, c->qmax );
//        pLog->Log("   gop_size : %i", c->gop_size );
//        pLog->Log("   min gop size : %i", c->keyint_min );
//        pLog->Log("   max_b_frames : %i", c->max_b_frames );
//        pLog->Log("   b_frame_strategy (best nb B-frames) : %i", c->b_frame_strategy );
//        pLog->Log("   refs : %i", c->refs );
//        pLog->Log("   i_quant_factor : %lf", c->i_quant_factor );
//
//    // is experimental?
////    if( m_pOutputCodec->capabilities & CODEC_CAP_EXPERIMENTAL )
////        pLog->Log("\t experimental : yes.\n");
////    else
////        pLog->Log("\t experimental : no.\n");
//
//    return;
//}

void ffmpegTools::print_info( AVStream *c )
{
	printf("print_info AVStream :\n");
	if( c->codec )
	{
		char buf[256];
		avcodec_string(buf, sizeof(buf), c->codec, 0);
		printf("\t %s\n", buf );
		print_info( c->codec );
	}
	else
	{
		printf("\t codec : not found\n" );
		return;
	}

	return;
}

void ffmpegTools::print_info( AVFrame *c, const char *name )
{

    printf("print_info AVFrame : %s\n", name );

//    printf("\t ref : %s\n", (c->==NULL)?"no":"yes" );
    printf("\t data : %s\n", (c->data == NULL)?"no":"yes" );
    printf("\t size : %ix%i\n", c->width, c->height );
    printf("\t linesize : %ix%ix%i\n", c->linesize[0], c->linesize[1], c->linesize[2] );
    printf("\t format : %i\n", c->format );
    printf("\t pts : %ld\n", c->pts );
    printf("\t pkt_pts : %ld\n", c->pkt_pts );
    printf("\t pkt_dts : %ld\n", c->pkt_dts );

    return;
}

//void ffmpegTools::print_info( AVCodec *c )
//{

//    printf("print_info AVCodec :\n");
//
//    printf("\t coder_type : %s\n", c->coder_type );
//    printf("\t nb_streams : %s\n", c->nb_streams );
//    printf("\t packet_size : %i\n", c->packet_size );
//    printf("\t bit_rate : %i\n", c->bit_rate );
//    printf("\t max_picture_buffer : %i\n", c->max_picture_buffer );
//
//    return;
//}

//int ffmpeg_prepare_frame(struct ffmpeg *ffmpeg, unsigned char *y,
//                              unsigned char *u, unsigned char *v)
//{
//	AVFrame *picture;
//
//	picture = avcodec_alloc_frame();
//	if (!picture) {
//		motion_log(LOG_ERR, 1, "Could not alloc frame");
//		return NULL;
//	}
//
//	/* take care of variable bitrate setting */
//	if (ffmpeg->vbr) {
//		picture->quality = ffmpeg->vbr;
//	}
//
//	/* setup pointers and line widths */
//	picture->data[0] = y;
//	picture->data[1] = u;
//	picture->data[2] = v;
//	picture->linesize[0] = ffmpeg->c->width;
//	picture->linesize[1] = ffmpeg->c->width / 2;
//	picture->linesize[2] = ffmpeg->c->width / 2;
//
//	return picture;
//}

void ffmpegTools::IplImage_to_AVFrame( const IplImage *iplImage, AVFrame *avFrame, uint8_t **pData, const int frameWidth, const int frameHeight, const enum PixelFormat pix_fmt)
{


    // setup convert ctx
    //
        struct SwsContext* img_convert_ctx = 0;
        //int linesize[4] = {0, 0, 0, 0};
        img_convert_ctx = sws_getContext( iplImage->width, iplImage->height, PIX_FMT_BGR24,
                                          frameWidth, frameHeight, PIX_FMT_YUV420P,
                                          SWS_BICUBIC, 0, 0, 0);
    // convert
    //
        if (img_convert_ctx != 0)
        {
                const uint8_t *pDataImage = (uint8_t *)iplImage->imageData;
                //const int lineSizeIpl = iplImage->widthStep;
                avFrame->linesize[0] = 3 * iplImage->width;

                printf( "%ix%i -> %ix%i (step=%i)\n", iplImage->width, iplImage->height, frameWidth, frameHeight, avFrame->linesize[0] );
                sws_scale(img_convert_ctx, &pDataImage, avFrame->linesize, 0, iplImage->height, avFrame->data, avFrame->linesize);
                sws_freeContext(img_convert_ctx);
        }

    return;
}

void ffmpegTools::IplImage_To_AVFrame( const IplImage *ipl_src, AVFrame *avFrame_dst, int mem_cpy )
{
    // src/dst sizes
        int w_src = ipl_src->width;
        int h_src = ipl_src->height;
        int w_dst = avFrame_dst->width;
        int h_dst = avFrame_dst->height;

    // src/dst data ptr / widthStep
        uint8_t *pData_dst = (uint8_t *)avFrame_dst->data;
        uint8_t *pData_src = (uint8_t *)ipl_src->imageData;

    // src/dst format
        const enum PixelFormat pix_fmt_src = (PixelFormat)PIX_FMT_BGR24;    // opencv IplImage format
        const enum PixelFormat pix_fmt_dst = (PixelFormat)avFrame_dst->format;

    // setup convert ctx
    //
        struct SwsContext* img_convert_ctx = 0;
        img_convert_ctx = sws_getContext( w_src, h_src, pix_fmt_src,
                                          w_dst, h_dst, pix_fmt_dst,
                                          SWS_BICUBIC, 0, 0, 0);
    // convert
    //
        if (img_convert_ctx != 0)
        {
                printf( "%ix%i -> %ix%i (step=%i -> %i) %i, %i\n", w_src, h_src, w_dst, h_dst, avFrame_dst->linesize[0], ipl_src->widthStep, avFrame_dst->format, pix_fmt_dst );
                sws_scale(img_convert_ctx, &pData_src, &ipl_src->widthStep, 0, h_src, &pData_dst, avFrame_dst->linesize);
                sws_freeContext(img_convert_ctx);
        }

    return;
}

void ffmpegTools::cvMat_To_AVFrame( const cv::Mat &img_src, AVFrame *avFrame_dst, int mem_cpy )
{
        IplImage ipl_src = img_src;

    // src/dst sizes
        int w_src = ipl_src.width;
        int h_src = ipl_src.height;
        int w_dst = avFrame_dst->width;
        int h_dst = avFrame_dst->height;

    // src/dst data ptr / widthStep
        uint8_t *pData_dst = (uint8_t *)avFrame_dst->data;
        uint8_t *pData_src = (uint8_t *)ipl_src.imageData;

    // src/dst format
        const enum PixelFormat pix_fmt_src = (PixelFormat)PIX_FMT_BGR24;    // opencv IplImage format
        const enum PixelFormat pix_fmt_dst = (PixelFormat)avFrame_dst->format;

    // setup convert ctx
    //
        struct SwsContext* img_convert_ctx = 0;
        img_convert_ctx = sws_getContext( w_src, h_src, pix_fmt_src,
                                          w_dst, h_dst, pix_fmt_dst,
                                          SWS_BICUBIC, 0, 0, 0);
    // convert
    //
        if (img_convert_ctx != 0)
        {
                printf( "%ix%i -> %ix%i (step=%i -> %i) %i, %i\n", w_src, h_src, w_dst, h_dst, avFrame_dst->linesize[0], ipl_src.widthStep, avFrame_dst->format, pix_fmt_dst );
                sws_scale(img_convert_ctx, &pData_src, &ipl_src.widthStep, 0, h_src, &pData_dst, avFrame_dst->linesize);
                sws_freeContext(img_convert_ctx);
        }

    return;
}


void ffmpegTools::AVFrame_To_IplImage( const AVFrame *avFrame_src, IplImage *ipl_dst, int mem_cpy )
{
    // src/dst sizes
        int w_src = avFrame_src->width;
        int h_src = avFrame_src->height;
        int w_dst = ipl_dst->width;
        int h_dst = ipl_dst->height;

    // dst data ptr / widthStep
        uint8_t *pData_dst = (uint8_t *)ipl_dst->imageData;

    // src/dst format
        const enum PixelFormat pix_fmt_src = (PixelFormat)avFrame_src->format;
        const enum PixelFormat pix_fmt_dst = (PixelFormat)PIX_FMT_BGR24;    // opencv IplImage format

    // setup convert ctx
    //
        struct SwsContext* img_convert_ctx = 0;
        img_convert_ctx = sws_getContext( w_src, h_src, pix_fmt_src,
                                          w_dst, h_dst, pix_fmt_dst,
                                          SWS_BICUBIC, 0, 0, 0);
    // convert
    //
        if (img_convert_ctx != 0)
        {
                printf( "%ix%i -> %ix%i (step=%i -> %i) %i, %i\n", w_src, h_src, w_dst, h_dst, avFrame_src->linesize[0], ipl_dst->widthStep, avFrame_src->format, pix_fmt_dst );
                sws_scale(img_convert_ctx, avFrame_src->data, avFrame_src->linesize, 0, h_src, &pData_dst, &ipl_dst->widthStep);
                sws_freeContext(img_convert_ctx);
        }

    return;
}

void ffmpegTools::AVFrame_To_cvMat( const AVFrame *avFrame_src, cv::Mat &img_dst, int mem_cpy )
{
        IplImage ipl_dst = img_dst;

    // src/dst sizes
        int w_src = avFrame_src->width;
        int h_src = avFrame_src->height;
        int w_dst = ipl_dst.width;
        int h_dst = ipl_dst.height;

    // dst data ptr / widthStep
        uint8_t *pDataImage = (uint8_t *)ipl_dst.imageData;

    // src/dst format
        const enum PixelFormat pix_fmt_src = (PixelFormat)avFrame_src->format;
        const enum PixelFormat pix_fmt_dst = (PixelFormat)PIX_FMT_BGR24;    // opencv IplImage format

    // setup convert ctx
    //
        struct SwsContext* img_convert_ctx = 0;
        img_convert_ctx = sws_getContext( w_src, h_src, pix_fmt_src,
                                          w_dst, h_dst, pix_fmt_dst,
                                          SWS_BICUBIC, 0, 0, 0);
    // convert
    //
        if (img_convert_ctx != 0)
        {
                printf( "%ix%i -> %ix%i (step=%i -> %i) %i, %i\n", w_src, h_src, w_dst, h_dst, avFrame_src->linesize[0], ipl_dst.widthStep, avFrame_src->format, pix_fmt_dst );
                sws_scale(img_convert_ctx, avFrame_src->data, avFrame_src->linesize, 0, h_src, &pDataImage, &ipl_dst.widthStep);
                sws_freeContext(img_convert_ctx);
        }

    return;
}

/**
 * the following function is a modified version of code
 * found in ffmpeg-0.4.9-pre1/output_example.c
 */
AVFrame *ffmpegTools::alloc_picture( int pix_fmt, int width, int height, bool alloc )
{
    AVFrame *av_frame;
    uint8_t *avframe_buf;
    int size;

    av_frame = avcodec_alloc_frame();
    if( !av_frame )
            return NULL;

    size = avpicture_get_size( (PixelFormat) pix_fmt, width, height );
    //determine required buffer size and allocate buffer
    avframe_buf = (uint8_t *)av_malloc(size);
    if (!avframe_buf) {
             av_free(av_frame);
             return NULL;
    }
    //assign appropriate parts of buffer to image planes in av_frame
    avpicture_fill( (AVPicture *)av_frame, avframe_buf, (PixelFormat)pix_fmt, width, height );

    return av_frame;
}

void ffmpegTools::free_pixture( AVFrame **picture)
{
    if((*picture)->data[0])
        free((*picture)->data[0]);

    av_free(*picture);

    return;
}

double ffmpegTools::r2d(AVRational r)
{
    return r.num == 0 || r.den == 0 ? 0. : (double)r.num / (double)r.den;
}

double ffmpegTools::get_fps( AVFormatContext *ic, int nVideoStream )
{
    double eps_zero=0.000025;
    double fps = r2d(ic->streams[nVideoStream]->r_frame_rate);

    if (fps < eps_zero)
    {
        fps = r2d(ic->streams[nVideoStream]->avg_frame_rate);
    }

    if (fps < eps_zero)
    {
        fps = 1.0 / r2d(ic->streams[nVideoStream]->codec->time_base);
    }

    return fps;
}

int64_t ffmpegTools::dts_to_frame_number(int64_t dts, AVFormatContext *ic, int nVideoStream)
{
    double sec = dts_to_sec(dts, ic, nVideoStream);
    return (int64_t)(ffmpegTools::get_fps(ic, nVideoStream) * sec + 0.5);
}

double ffmpegTools::dts_to_sec(int64_t dts, AVFormatContext *ic, int nVideoStream)
{
    return (double)(dts - ic->streams[nVideoStream]->start_time) * r2d(ic->streams[nVideoStream]->time_base);
}

int count = 0;
double sum = 0;
double h;
double w;
bool dbg=true;

void print_vector(int x, int y, int dx, int dy)
{
	if (dx != 10000 && dy != 10000){
		sum = sum + sqrt((double)dx*(double)dx/w/w + (double)dy*(double)dy/h/h);
		count++;
	}
    if(dbg)
        // printf("(%d %d):(%d %d)\t", x, y, dx, dy);
        printf("(%d %d)\t", dx, dy);
}

/* Print motion vector for each macroblock in this frame.  If there is
 * no motion vector in some macroblock, it prints a magic number NO_MV. */
//void ffmpegTools::printMVMatrix(int index, AVFrame *pict, AVCodecContext *ctx, int *pMvData )
//{
//    const int mb_width  = (ctx->width + 15) / 16;
//    const int mb_height = (ctx->height + 15) / 16;
//    const int mb_stride = mb_width + 1;
//    const int mv_sample_log2 = 4 - pict->motion_subsample_log2;
//    const int mv_stride = (mb_width << mv_sample_log2) + (ctx->codec_id == CODEC_ID_H264 ? 0 : 1);
//    const int quarter_sample = (ctx->flags & CODEC_FLAG_QPEL) != 0;
//    const int shift = 1 + quarter_sample;
//
//    if(dbg)
//        printf("frame %d, %d x %d\n", index, mb_height, mb_width);
//
//    for (int mb_y = 0; mb_y < mb_height; mb_y++)
//    {
//        for (int mb_x = 0; mb_x < mb_width; mb_x++)
//        {
//            const int mb_index = mb_x + mb_y * mb_stride;
//            const int dx_index = 2*(mb_x + mb_y * mb_stride);
//            const int dy_index = dx_index+1;
//            if (pict->motion_val)
//            {
//                for (int type = 0; type < 3; type++)
//                {
//                    int direction = 0;
//                    switch (type) {
//                         case 0:
//                             if ((!(ctx->debug_mv & FF_DEBUG_VIS_MV_P_FOR)) || (pict->pict_type!= AV_PICTURE_TYPE_P))
//                                 continue;
//                             direction = 0;
//                             break;
//                         case 1:
//                             if ((!(ctx->debug_mv & FF_DEBUG_VIS_MV_B_FOR)) || (pict->pict_type!= AV_PICTURE_TYPE_B))
//                                 continue;
//                             direction = 0;
//                             break;
//                         case 2:
//                             if ((!(ctx->debug_mv & FF_DEBUG_VIS_MV_B_BACK)) || (pict->pict_type!= AV_PICTURE_TYPE_B))
//                                 continue;
//                             direction = 1;
//                             break;
//                         }
//                    /*
//                    switch (type)
//                    {
//                        case 0:
//                            if (pict->pict_type != AV_PICTURE_TYPE_P)
//                                continue;
//                            direction = 0;
//                            break;
//                        case 1:
//                            if (pict->pict_type != AV_PICTURE_TYPE_B)
//                                continue;
//                            direction = 0;
//                            break;
//                        case 2:
//                            if (pict->pict_type != AV_PICTURE_TYPE_B)
//                                continue;
//                            direction = 1;
//                            break;
//                    }
//                    */
//
//                    if (!USES_LIST(pict->mb_type[mb_index], direction))
//                    {
//#define NO_MV 10000
//                        if (IS_8X8(pict->mb_type[mb_index]))
//                        {
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//
//                        }
//                        else if (IS_16X8(pict->mb_type[mb_index]))
//                        {
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                        }
//                        else if (IS_8X16(pict->mb_type[mb_index]))
//                        {
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                        }
//                        else
//                        {
//                            print_vector(mb_x, mb_y, NO_MV, NO_MV);
//                            //pMvData[dx_index] = pMvData[dy_index] = NO_MV;
//                        }
//#undef NO_MV
//                        continue;
//                    }
//
//                    if (IS_8X8(pict->mb_type[mb_index]))
//                    {
//                        for (int i = 0; i < 4; i++)
//                        {
//                            int xy = (mb_x*2 + (i&1) + (mb_y*2 + (i>>1))*mv_stride) << (mv_sample_log2-1);
//                            int dx = (pict->motion_val[direction][xy][0]>>shift);
//                            int dy = (pict->motion_val[direction][xy][1]>>shift);
//                            print_vector(mb_x, mb_y, dx, dy);
//                            pMvData[dx_index] = dx;
//                            pMvData[dy_index] = dy;
//                        }
//                    }
//                    else if (IS_16X8(pict->mb_type[mb_index]))
//                    {
//                        for (int i = 0; i < 2; i++)
//                        {
//                            int xy = (mb_x*2 + (mb_y*2 + i)*mv_stride) << (mv_sample_log2-1);
//                            int dx = (pict->motion_val[direction][xy][0]>>shift);
//                            int dy = (pict->motion_val[direction][xy][1]>>shift);
//
//                            if (IS_INTERLACED(pict->mb_type[mb_index]))
//                                dy *= 2;
//
//                            print_vector(mb_x, mb_y, dx, dy);
//                            pMvData[dx_index] = dx;
//                            pMvData[dy_index] = dy;
//                        }
//                    }
//                    else if (IS_8X16(pict->mb_type[mb_index]))
//                    {
//                        for (int i = 0; i < 2; i++)
//                        {
//                            int xy =  (mb_x*2 + i + mb_y*2*mv_stride) << (mv_sample_log2-1);
//                            int dx = (pict->motion_val[direction][xy][0]>>shift);
//                            int dy = (pict->motion_val[direction][xy][1]>>shift);
//
//                            if (IS_INTERLACED(pict->mb_type[mb_index]))
//                                dy *= 2;
//
//                            print_vector(mb_x, mb_y, dx, dy);
//                            pMvData[dx_index] = dx;
//                            pMvData[dy_index] = dy;
//                        }
//                    }
//                    else
//                    {
//                        int xy = (mb_x + mb_y*mv_stride) << mv_sample_log2;
//                        int dx = (pict->motion_val[direction][xy][0]>>shift);
//                        int dy = (pict->motion_val[direction][xy][1]>>shift);
//                        print_vector(mb_x, mb_y, dx, dy);
//                        pMvData[dx_index] = dx;
//                        pMvData[dy_index] = dy;
//                    }
//                }
//            }
////            if(dbg)
////            printf("--\n");
//        }
//        if(dbg)
//            printf("====\n");
//    }
//    if(dbg)
//            printf("end\n");
//}


void ffmpegTools::drawMVMatrix( cv::Mat &img, int *pMvData )
{
    const int mb_width  = (img.cols+15)/16;
    const int mb_height = (img.rows+15)/16;
    //int *pData=NULL;
    cv::Scalar color;

    // parse tab
    for( int y = 0; y<mb_height; ++y)
    {
        for( int x=0; x<mb_width; ++x )
        {
            const int index = 2*(x + y*mb_width);
            //const int pos_x = 4*x-2;        // image x4, -2 pour prendre au centre
            //const int pos_y = 4*y-2;

            int dx = pMvData[index];
            int dy = pMvData[index+1];

            if( (dx!=0) || (dy!=0) )
                print_vector(x, y, dx, dy);
            /*
              dx = ...;
              dy = ...;
              color = getColor( dy, dx );
              drawArraw( img, x, y, dx, dy );
            */
        }
    }
}

/* Take an image as character data and write it out to
 * the pFrame variable.  Also manage the conversion from
 * BGR to YUV color formats.  This function is mostly
 * for internal use and should normally should not be
 * called directly.  Use video_encode_frame instead.
 */

//void video_data_to_frame(video_t *video, char *data)
//{
//	AVPicture pic;
//	int width = video->pCodecCtx->width;
//	int height = video->pCodecCtx->height;
//
//	avpicture_fill((AVPicture*)video->pFrame, video->frame_buffer, PIX_FMT_YUV420P, width, height);
//	avpicture_fill(&pic, data, PIX_FMT_BGR24, width, height);
//	img_convert((AVPicture*)video->pFrame, PIX_FMT_YUV420P, &pic,
//				PIX_FMT_BGR24, width, height);
//}

/* Take an image as character data and write it out to
 * the video file.  This is the preferred method of
 * working with image data and should be used instead
 * of calling video_data_to_frame directly.
 */

//void video_encode_frame(video_t *video, char *data)
//{
//	int outsize;
//
//	video_data_to_frame(video, data);
//
//	outsize = avcodec_encode_video(video->pCodecCtx, video->buffer,
//				video->numBytes, video->pFrame);
//
//	fwrite(video->buffer, outsize, 1, video->file);
//}
