#ifndef FFMPEGTOOLS_H_INCLUDED
#define FFMPEGTOOLS_H_INCLUDED

#include <stdio.h>

extern "C"
{
//#define __STDC_CONSTANT_MACROS
#define __STDC_LIMIT_MACROS
#include "libavformat/avformat.h"
#include "libavutil/avutil.h"
#include "libavutil/opt.h"
#include "libavcodec/avcodec.h"
#include "libavutil/avstring.h"
#include "libswscale/swscale.h"

}

//#include "../tools/vizLog.h"

#include <opencv2/core/core.hpp>
#include <opencv2/core/core_c.h>
#include <opencv2/imgproc/imgproc.hpp>


#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>

#define IS_INTERLACED(a) ((a)&MB_TYPE_INTERLACED)
#define IS_16X16(a)      ((a)&MB_TYPE_16x16)
#define IS_16X8(a)       ((a)&MB_TYPE_16x8)
#define IS_8X16(a)       ((a)&MB_TYPE_8x16)
#define IS_8X8(a)        ((a)&MB_TYPE_8x8)
#define USES_LIST(a, list) ((a) & ((MB_TYPE_P0L0|MB_TYPE_P1L0)<<(2*(list))))

#define PRINT_LIB_INFO(libname, LIBNAME){                                \
             unsigned int version = libname##_version();                 \
             printf("%slib%-9s %2d.%3d.%2d / %2d.%3d.%2d\n",\
                    " ", #libname,                                    \
                    LIB##LIBNAME##_VERSION_MAJOR,                        \
                    LIB##LIBNAME##_VERSION_MINOR,                        \
                    LIB##LIBNAME##_VERSION_MICRO,                        \
                    version >> 16, version >> 8 & 0xff, version & 0xff); \
                    }

namespace ffmpegTools
{
    void print_info( AVCodecContext *c );
    //void print_info( AVCodecContext *c, vizILogger *pLog );
    void print_info( AVFormatContext *c );
    //void print_info( AVCodec *c );
    void print_info( AVStream *c );
    void print_info( AVPacket *c );

    void print_info( AVFrame *c, const char *name=NULL );

    void AVFrame_To_IplImage( const AVFrame *avFrame_src, IplImage *ipl_dst, int mem_cpy );
    void AVFrame_To_cvMat( const AVFrame *avFrame_src, cv::Mat &img_dst, int mem_cpy );
    void IplImage_To_AVFrame( const IplImage *ipl_src, AVFrame *avFrame_dst, int mem_cpy );
    void cvMat_To_AVFrame( const cv::Mat &img_src, AVFrame *avFrame_dst, int mem_cpy );

    void IplImage_to_AVFrame( const IplImage *iplImage, AVFrame *avFrame, uint8_t **pData, const int frameWidth, const int frameHeight, const enum PixelFormat pix_fmt=PIX_FMT_YUV420P );
//    void AVFrame_To_IplImage( const AVFrame *avFrame, const uint8_t *pData, IplImage *iplImage, const int frameWidth, const int frameHeight, const enum PixelFormat pix_fmt=PIX_FMT_BGR24);
    AVFrame *alloc_picture( int pix_fmt, int width, int height, bool alloc );
    void free_pixture( AVFrame **picture);

    double r2d(AVRational r);
    double get_fps( AVFormatContext *ic, int nVideoStream );

    int64_t dts_to_frame_number(int64_t dts, AVFormatContext *ic, int nVideoStream);
    double  dts_to_sec(int64_t dts, AVFormatContext *ic, int nVideoStream);
//    void printMVMatrix(int index, AVFrame *pict, AVCodecContext *ctx, int *pMvData);
    void drawMVMatrix( cv::Mat &img, int *pMvData);

    std::string getStringError( int error );

    void getRGBColor(const AVFrame& frame, int x, int y, unsigned char &r, unsigned char &g, unsigned char &b);

}


#endif // FFMPEGTOOLS_H_INCLUDED


/*

PIX_FMT_YUV420P contains three channels (or components):

    Y' (luminance) channel
    Cb (chrominance blue) channel
    Cr (chrominance red) channel.

The chrominance is subsampled using the 420 scheme:

Note: Be careful not to confuse this pixel format with PIX_FMT_YUVJ420P.

YCbCr 4:2:0 indicates a planar YCbCr colorspace where a 2x2 block of pixels in an image
is represented by 4 Y samples, 1 for each pixel, but all 4 pixels share a Cb sample
and a Cr sample.

Note: The P in YUV420P indicates planar not progressive or palette or packed.

It has the following characteristics:

    YUV colorspace.
    12bits per pixel average.

*/
