#include <signal.h>

#define BOOST_ALL_DYN_LINK
#include <boost/log/trivial.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>

#include "data.hpp"
#include "Capture.hpp"
#include "FluxVideoModifie.hpp"

static const cv::String keys =
//pas de tabulation!!!
	"{ help       |  | Affiche l'aide}"
	"{ cam-url    |  | url camera}"
	"{ cam-id     |1 | id de la camera}"
	"{ out-width  |0 | largeur image sortie}"
	"{ out-height |0 | hauteur image sortie}"
	"{ opencv     |  | transforme l'image au format opencv}"
	"{ kill       |  | tue la memoire partagee ayant "
			"le numero cam-id au lieu de la creer.}"
;


class ArgumentsSHMServer
{
public:
	bool opencv;
	int width;
	int height;
	int camId;
	std::string url;
};

//definition
void SIGINT_handler(int);	/* for SIGINT	*/
void SIGQUIT_handler(int);	/* for SIGQUIT	*/
ArgumentsSHMServer* parseArgument(int argc, const char* const argv[]);
void killSHM(ArgumentsSHMServer*);
void remplirSHM(data*, uint8_t, int);
//fin definition

static std::string nomSHM;

int main(int argc, char *argv[])
{
	using namespace boost::interprocess;

	/**
	 * Extraction des donnees entrees par l'utilisateur.
	 */
	ArgumentsSHMServer* options = parseArgument(argc, argv);
	
	/**
	 * Initialisation du call back quit.
	 */
	if( signal(SIGINT, SIGQUIT_handler) == SIG_ERR)
	{
		printf("SIGKILL install error\n");
		exit(2);
	}

	/**
	 * Remove shared memory on construction and destruction.
	 */
	struct shm_remove{
		shm_remove()	{shared_memory_object::remove(nomSHM.c_str());}
		~shm_remove()	{shared_memory_object::remove(nomSHM.c_str());}
	}remover;

	/**
	 * Initialisation de la memoire partagee.
	 */
	size_t tailleSHM = options->width * options->height * 3 * 8;
	nomSHM = genereNomSHM(options->camId);
	printf("Taille de la memoire partagee : %d\n", (int)tailleSHM);
	printf("hauteur : %d largeur : %d\n", options->height, options->width);
	managed_shared_memory* segment;
	data* instance;
	try{
		segment = new managed_shared_memory(create_only, nomSHM.c_str(), tailleSHM);

		data d;
		d.width =		options->width;
		d.height =		options->height;
		d.nbCanaux =	3;
		d.numImage =	0;
		remplirSHM(&d, 255, tailleSHM / 8);
		instance = segment->construct<data>
				("video")//name of the object
				(d);
	}catch(boost::interprocess::interprocess_exception e){
		BOOST_LOG_TRIVIAL(debug) << "ServerSHM.cpp " <<
				e.what();
		exit(EXIT_FAILURE);
	}
	remplirSHM(instance, 125, tailleSHM / 8);

	/**
	 * Initialisation de la video.
	 */
	Capture video(options->url);
	
	FluxVideoModifie* fm;
	enum PixelFormat formatPixel = PIX_FMT_BGR24;
	fm = new FluxVideoModifie(&video, options->width, options->height, formatPixel);
	// if(options->opencv){
	// 	//cette option est activee avec l'option --opencv
	// 	enum PixelFormat formatPixel = PIX_FMT_BGR24;
	// 	fm = new FluxVideoModifie(&video, options->width, options->height, formatPixel);
	// }else{
	// 	fm = new FluxVideoModifie(&video, options->width, options->height);
	// }
	
	// si la video n'est pas correcte on le fait savoir
	if(!video.estCorrect()){
		remplirSHM(instance, 255, tailleSHM / 8);
		printf("la video n'est pas correcte.\n");
	}

	/**
	 * Boucle de mise a jour.
	 */
	printf("Debut boucle de lecture\n");
	// int tailleEcriture = 1;
	int tailleEcriture = tailleSHM / 8;
	while(true){
		if(!video.lire()){
			remplirSHM(instance, 125, tailleSHM / 8);
			break;
		}
		if(!fm->modifier()){
			remplirSHM(instance, 125, tailleSHM / 8);
			break;
		}

		if(instance == NULL)
			printf("instance est null\n");
		++(instance->numImage);
		printf("numImage %d\n", instance->numImage);
		memcpy(instance->data, 
				fm->getPtrDataModifiees()->data[0], 
				tailleEcriture);
		sleep(0.01);
	}

	/**
	 * Destructeur du programme.
	 */
	shared_memory_object::remove(nomSHM.c_str());
	delete fm;
	delete options;
	delete segment;
}

void  SIGQUIT_handler(int sig)
{
	signal(sig, SIG_IGN);
	
	printf("Fin du server. Destruction de la memoire partagee\n");
	boost::interprocess::shared_memory_object::remove(nomSHM.c_str());

	exit(3);
}


ArgumentsSHMServer* parseArgument(int argc, const char* const argv[])
{
	cv::CommandLineParser parser(argc, argv, keys);
	ArgumentsSHMServer* d = new ArgumentsSHMServer();

	if(parser.has("help"))
		parser.printMessage();
	d->opencv	= parser.has("opencv");
	d->url		= parser.get<std::string>("cam-url");
	d->width	= parser.get<int>("out-width");
	d->height	= parser.get<int>("out-height");
	d->camId	= parser.get<int>("cam-id");

	if(parser.has("kill")){
		killSHM(d);
		exit(EXIT_SUCCESS);
	}

	if (!parser.check()){
		parser.printErrors();
	}

	//on modifie les options si besoin est
	if(d->width <= 0)
		d->width = largeurStd;
	if(d->height <= 0 )
		d->height = hauteurStd;

	return d;
}

void killSHM(ArgumentsSHMServer* arg)
{
	std::string str = genereNomSHM(arg->camId);
	printf("Fermer la memoire : %s\n", str.c_str());
	boost::interprocess::shared_memory_object::remove(str.c_str());
}

void remplirSHM(data* d, uint8_t couleur, int taille)
{
	memset(d->data,
			couleur, 
			taille);
}