/*
 * FluxVideo.cpp
 *
 *  Created on: Jun 6, 2013
 *      Author: exploit
 *
 * Classe auquel on donne un url et permet d'obtenir le flux encodé et décodé.
 * Utilise FFMpeg.
 */

#ifndef FLUXVIDEO_HPP_
#define FLUXVIDEO_HPP_

#include <vector>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <inttypes.h>

extern "C" {
#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libavutil/imgutils.h>
#include <libavutil/samplefmt.h>
#include <libavutil/avutil.h>
#include <libavutil/time.h>
#include <libavutil/imgutils.h>
#include <libavutil/opt.h>
#include <libswscale/swscale.h>
}
#define BOOST_ALL_DYN_LINK
#include <boost/log/trivial.hpp>

#include "ffmpegTools.hpp"
#include "CallBackParam.hpp"


//TODO définir le nom de la classe
class Capture
{
public:
	Capture(const std::string &);
	Capture(const Capture &);
	virtual ~Capture();

	std::string getUrl() const;
	int getHauteur()const;
	int getLargeur()const;
	enum PixelFormat getFormat()const;
	double getFPS()const;
	int getFramSize()const;
	bool estCorrect()const;
	AVFormatContext* getFluxFFmpeg()const;
	AVCodecContext* getDecodeContext()const;
	AVPacket* getDataEncode()const;
	AVFrame* getDataDecode()const;

	bool lire();

protected:
	//initialisation
	void init();
	void initFluxFFmpeg();
	void initStreamVideo();
	void initAlloueAttribut();

	//traitement
	bool decodeVideo();
	bool lirePacket();

	/*
	 * url qui permet de récupérer le flux vidéo . Le protocole de transmission
	 * doit être rtsp.
	 */
	std::string url;

	/*
	 * Vrai si le traitement du flux vidéo s'est déroulé sans problème, faux sinon.
	 */
	bool correct;

	/*
	 * Structure qui représente le flux vidéo pour FFMpeg.
	 */
	AVFormatContext* fluxFFmpeg;

	/*
	 * Frame vidéo encodé.
	 */
	AVPacket* packetEncode;

	/*
	 * Frame vidéo décodé.
	 */
	AVFrame* frameDecode;

	/*
	 * AVStream contenant la vidéo.
	 */
	AVStream* streamVideo;

	/*
	 * Structure qui détermine le fonctionnement de la fonction de callback associée
	 * à fluxFFmpeg.
	 * Mettre cbp->actif = false pour que la fonction de callback n'arrete pas
	 * la capture quoi qu'il arrive.
	 * Mettre cbp->start = av_gettime() pour remettre le décompte à zéros.
	 */
	CallBackParam* cbp;
};



/*cv::Mat img(frameDecode->height,
			frameDecode->width,
			CV_8UC3,
			(uchar*)bufferRGB->data
			);*/

//static void pgm_save(unsigned char *buf, int wrap, int xsize, int ysize,
//				 char *filename)
//{
//	FILE *f;
//	int i;
//
//	f=fopen(filename,"w");
//	fprintf(f,"P5\n%d %d\n%d\n",xsize,ysize,255);
//	for(i=0;i<ysize;i++)
//		fwrite(buf + i * wrap,1,xsize,f);
//	fclose(f);
//}

#endif /* FLUXVIDEO_HPP_ */
