/*
 * FluxVideoModifie.cpp
 *
 *  Created on: Jun 12, 2013
 *      Author: exploit
 */

#include "FluxVideoModifie.hpp"

/*******************************************************************************
								CONSTRUCTEUR/DESTRUCTEUR
*******************************************************************************/

/*
 * Constructeurs qui définit une sortie qui sera une modification à partir
 * de la vidéo décodée. Cette sortie sera aceessible via getDataModifiees
 * @theUrl		addresse de la source vidéo.
 * @largueur	largueur à laquelle il faudra redimentionner la vidéo en
 * 				nombre de pixel
 * @hauteur		hauteur à laquelle il faudra redimentionner la vidéo en
 * 				nombre de pixel
 * @format		format des pixel pour la sortie modifiée.
 * 				Pour créer une image OpenCV, le PixelFormat = PIX_FMT_BGR24 convient.
 */
FluxVideoModifie::FluxVideoModifie(Capture* theFlux, int largueur, int hauteur,
		enum PixelFormat format)
		: model(theFlux), largueurSortie(largueur), hauteurSortie(hauteur),
		  formatSortie(format)
{
	frameModifie = initFrameModification();
	initSwsContext();
}

/*
 * Constructeurs qui définit une sortie qui sera une modification à partir
 * de la vidéo décodée. Cette sortie sera aceessible via getDataModifiees
 * @theUrl		addresse de la source vidéo.
 * @largueur 	largueur à laquelle il faudra redimentionner la vidéo en
 * 				nombre de pixel
 * @hauteur		hauteur à laquelle il faudra redimentionner la vidéo en
 * 				nombre de pixel
 */
FluxVideoModifie::FluxVideoModifie(Capture *theFlux, int largueur, int hauteur)
		: model(theFlux), largueurSortie(largueur), hauteurSortie(hauteur),
		  formatSortie(theFlux->getFormat())
{
	frameModifie = initFrameModification();
	initSwsContext();
}

/*
 * Constructeurs qui définit une sortie qui sera une modification à partir
 * de la vidéo décodée. Cette sortie sera aceessible via getDataModifiees
 * @theUrl 	addresse de la source vidéo.
 * @format	format des pixel pour la sortie modifiée.
 * 			Pour créer une image OpenCV, le PixelFormat = PIX_FMT_BGR24 convient.
 */
FluxVideoModifie::FluxVideoModifie(Capture* theFlux, enum PixelFormat format)
		: model(theFlux), largueurSortie(theFlux->getLargeur()),
		  hauteurSortie(theFlux->getHauteur()), formatSortie(format)
{
	frameModifie = initFrameModification();
	initSwsContext();
	//ffmpegTools::print_info(theFlux->getDataDecode(), "theFlux->getDataDecode()");
	//ffmpegTools::print_info(frameModifie, "frameModifie");
}

FluxVideoModifie::FluxVideoModifie(const FluxVideoModifie &theFlux)
		: model(theFlux.model),
		  contextTraitementImage(theFlux.contextTraitementImage),
		  largueurSortie(theFlux.largueurSortie), hauteurSortie(theFlux.hauteurSortie),
		  formatSortie(theFlux.formatSortie), frameModifie(theFlux.frameModifie)
{

}

FluxVideoModifie::~FluxVideoModifie()
{
	free(frameModifie);
}

/*******************************************************************************
								FONCTIONS PUBLIQUES
*******************************************************************************/

/*
 * Retourne la largueur (nombre de colonne) de l'image.
 */
int FluxVideoModifie::getLargeur()const
{
	return largueurSortie;
}

/*
 * Retourne la hauteur (nombre de ligne) de l'image.
 */
int FluxVideoModifie::getHauteur()const
{
	return hauteurSortie;
}

enum PixelFormat FluxVideoModifie::getFormat()const
{
	return formatSortie;
}

/*
 * Retourne la taille d'un paquet cree par cette classe.
 * TODO Fonctionne ?
 */
int FluxVideoModifie::getSize()const
{
	return tailleSortie;
	// return 	frameModifie->linesize * frameModifie->height;
}


/***
 * Permet d'obtenir le pointeur vers les données modifiées. Ne garantie pas que les données
 * le AVFrame->data ne soit pas NULL.
 * @return	Retourne le pointeur vers le AVFrame contenant les données modifiées.
 */
AVFrame* FluxVideoModifie::getPtrDataModifiees()const
{
	return frameModifie;
}

/***
 * Modifie les données contenues dans le paquet passé en paramètre du constructeur.
 * @return	Vrai si la modification s'est bien déroulée.
 */
bool FluxVideoModifie::modifier()
{
	if(!model->estCorrect())
			return false;

	sws_scale(contextTraitementImage,
			model->getDataDecode()->data,//entrée FFMpeg
			model->getDataDecode()->linesize,
			0,//on modifie l'intégralité de l'image
			model->getHauteur(),
			frameModifie->data,//sortie RGB24 compatible OpenCV
			frameModifie->linesize);

	//On vérifique que la conversion a produit quelques chose
	if(frameModifie->data == NULL){
		fprintf(stderr, "FluxVideo::modifierFluxDecode la conversion a échouée.\n");
		return false;
	}
	return true;

//	ffmpegTools::print_info(frameModifie, "RGB_image");
//
//	//dataRGB = bufferRGB;
//
//	printf("###FIN FluxVideo::modifierFluxDecode###\n");
}

/*******************************************************************************
								FONCTIONS PROTECTED
									INITIALISATION
*******************************************************************************/
/*
 * Initialise le SwsContext qui servira à modifier le flux décodé pour avoir
 * une sortie à un format spécifique.
 * Pour créé une image OpenCV, le PixelFormat = PIX_FMT_BGR24 convient.
 */
void FluxVideoModifie::initSwsContext()
{
	if(!model->estCorrect())
		return;

	//On créé le SwsContext
	contextTraitementImage = sws_getContext(model->getLargeur(),
			model->getHauteur(),
			model->getFormat(),
			largueurSortie,
			hauteurSortie,
			formatSortie,
			/*
			 * algo utilisé pour la conversion.
			 * Les valeurs possibles pour ce paramètre se trouvent dans la partie
			 * macro de swscale.h
			 */
			SWS_BICUBIC,
			NULL, NULL, NULL);
//	if(formatSortie == PIX_FMT_BGR24){
//		printf("formatOpenCV\n");
//	}else{
//		printf("format non OpenCV");
//	}
//	printf("hauteur source %d \n", model->getHauteur());
//	printf("largueur source %d \n", model->getLargueur());
//	printf("hauteur dst %d \n", hauteurSortie);
//	printf("largueur dst %d \n", largueurSortie);
}

/*
 * Créé un frame formaté pour recevoir des données au format définie
 * par contextTraitementImage, à partir d'un model contenant un flux FFmpeg.
 */
AVFrame* FluxVideoModifie::initFrameModification()
{
	AVFrame* toReturn;

	// Allocate an AVFrame structure
	toReturn = avcodec_alloc_frame();
	if(toReturn == NULL){
		fprintf(stderr, "FluxVideo::initFrameRGB impossible d'allouer la mémoire.\n");
		return NULL;
	}

	// Determine required buffer size and allocate buffer
	tailleSortie = avpicture_get_size(
			formatSortie,
			largueurSortie,
			hauteurSortie);
	uint8_t* dataBuffer = (uint8_t*)av_malloc(tailleSortie * sizeof(uint8_t));

	// Assign appropriate parts of buffer to image planes in pFrameRGB
	avpicture_fill((AVPicture *)toReturn,
			dataBuffer,
			formatSortie,
			largueurSortie,
			hauteurSortie);

	toReturn->width = largueurSortie;
	toReturn->height = hauteurSortie;
	toReturn->format = formatSortie;

	return toReturn;
}
